/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   sampling_rate.c                                                         */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   Selecting the sampling rate number of the audio signal on the           */
/*   TMS320C5416 DSK.                                                        */
/*                                                                           */
/*   Takes an input from a microphone or from a CD player and feeds straight */
/*   through to loudspeakers. The user switches select the sampling rate     */
/*   between 2 kHz and 48 kHz.                                               */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00	                                                         */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*                                                                           */
/*   Revision 1.01															 */
/*   24th November 2002. Sampling rate increased from 24 kHz to 48 kHz.      */
/*                                                                           */
/*   Revision 1.00                                                           */
/*   2nd November 2002. Created by Richard Sikora.                           */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>    /* Required for functions printf() and puts()          */

/*****************************************************************************/
/*                                                                           */
/* The following line takes the name of the module, here sampling_bits.c and */
/* adds the letters cfg.h                                                    */
/*                                                                           */
/*****************************************************************************/

#include "sampling_ratecfg.h"

#include "dsk5416.h"
#include "dsk5416_pcm3002.h"

#include "bargraph.h"
#include "switches.h"


/*****************************************************************************/
/* Configuration setup of registers of PCM3002 Codec                         */
/*****************************************************************************/

DSK5416_PCM3002_Config setup = {

    0x1FF,      // Set-Up Reg 0 - Left channel DAC attenuation
    0x1FF,      // Set-Up Reg 1 - Right channel DAC attenuation
    0x0,        // Set-Up Reg 2 - Various ctl e.g. power-down modes
    0x0         // Set-Up Reg 3 - Codec data format control
};

/*****************************************************************************/
/*                                                                           */
/* For compatibility with pcm3002 read / write, these variables must         */
/* be declared as Int16 or short int, rather than int.                       */
/*                                                                           */
/*****************************************************************************/

Int16 left_input;
Int16 left_output; 
Int16 right_input;
Int16 right_output; 

/*****************************************************************************/
/* UserTask()                                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* The main user task.                                                       */
/*                                                                           */
/* Note that this task is not called by main(). It is scheduled by DSP/BIOS  */
/*                                                                           */
/*****************************************************************************/
 
void UserTask()
{
    DSK5416_PCM3002_CodecHandle hCodec;
    long i;
    unsigned int switch_value; 
    unsigned int counter = 0;
              
    /* Start the codec */
    hCodec = DSK5416_PCM3002_openCodec(0, &setup);
 
    /* Display project details on StdOut. \n is important. */
 
    puts("TMS320C5416 DSK: Changing the sampling rate between 2 kHz and 48 kHz.\n");
          
    for (i = 0; i < 12000000; i++)
    {
      /* Read left input channel */    
      while (!DSK5416_PCM3002_read16(hCodec, &left_input));

      /* Write to left output channel */      
      while (!DSK5416_PCM3002_write16(hCodec, left_output));
      
      /* Read right input channel */
      while (!DSK5416_PCM3002_read16(hCodec, &right_input));
      
      /* Write to right output channel */
      while (!DSK5416_PCM3002_write16(hCodec, right_output));
      
      /* Read user switches on DSK and display their meaning on Stdout. */
      switch_value = switch_status_display();
   
      counter++; /* Increment counter every time through */
  
      if ( 0 == switch_value)
       {
         /* 48000 Hz sampling rate */
         left_output = left_input;  
         right_output = right_input;
       }
      else if ( 1 == switch_value)
       {
         /* 24000 Hz sampling rate. Use 48000 Hz divided by 2 */
         
         if ( 0 == ( counter % 2) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }   
       }  
      else if ( 2 == switch_value)
       {
         /* 16000 Hz sampling rate. Use 48000 Hz divided by 3 */
         
         if ( 0 == ( counter % 3) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }           
       } 
      else if ( 3 == switch_value)
       {
         /* 12000 Hz sampling rate. Use 48000 Hz divided by 4 */
         
         if ( 0 == ( counter % 4) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }         
       }       
      else if ( 4 == switch_value)
       {
         /* 9600 Hz sampling rate. Use 48000 Hz divided by 5 */
         
         if ( 0 == ( counter % 5) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }        
       }         
      else if ( 5 == switch_value)
       {
         /* 8000 Hz sampling rate. Use 48000 Hz divided by 6 */
         
         if ( 0 == ( counter % 6) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }         
       }   
      else if ( 6 == switch_value)
       {
         /* 6800 Hz sampling rate. Use 48000 Hz divided by 7 */
         
         if ( 0 == ( counter % 7) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }  
       }       
      else if ( 7 == switch_value)
       {
         /* 6000 Hz sampling rate. Use 48000 Hz divided by 8 */
         
         if ( 0 == ( counter % 8) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }         
       }        
      else if ( 8 == switch_value)
       {
         /* 5300 Hz sampling rate. Use 48000 Hz divided by 9 */
         
         if ( 0 == ( counter % 9) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }          
       }        
      else if ( 9 == switch_value )
       {
         /* 4800 Hz sampling rate. Use 48000 Hz divided by 10 */
         
         if ( 0 == ( counter % 10) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }          
       } 
      else if ( 10 == switch_value)
       {
         /* 4000 Hz sampling rate. Use 48000 Hz divided by 12 */
         
         if ( 0 == ( counter % 12) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }            
       } 
      else if ( 11 == switch_value)
       {
         /* 3000 Hz sampling rate. Use 48000 Hz divided by 16 */
         
         if ( 0 == ( counter % 16) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           }         
       }
      else if ( 12 == switch_value)
       {
         /* 2000 Hz sampling rate. Use 48000 Hz divided by 24 */
         
         if ( 0 == ( counter % 24) )
           {
             left_output = left_input;  
             right_output = right_input;
           }         
         else
           {
             /* Not time to update yet */
           } 
       }        
      else if ( 13 == switch_value)
       {
          /* Not used */
       }
      else if ( 14 == switch_value)
       {
          /* Not used */
       }        
      else if ( 15 == switch_value)
       {
          /* Not used */
       }               
  
      /* Display greater of two outputs on bargraph */
      bargraph_6dB ( left_output, right_output);
     }

    /* Finished processing. Close the codec */
    DSK5416_PCM3002_closeCodec(hCodec);
    
    puts("TMS320C5416 DSK program has terminated.\n");
}

/*****************************************************************************/
/* main()                                                                    */
/*****************************************************************************/

void main()
{
    /* Initialize the board support library                         */
    /* There is no need to initialize the DIP switches and the LEDs */
    
    DSK5416_init();
    
    /* All other functions are scheduled by DSP/BIOS */
}

/******************************************************************************/
/* End of sampling_rate.c                                                     */
/******************************************************************************/

